@@ -1,9 +1,23 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * callbacks.c
+ *  Authors: Iain Holmes <iain@ximian.com>
+ *
+ *  Copyright 2001 Iain Holmes
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- * Copyright (C) 2001 Iain Holmes
- * Authors: Iain Holmes  <iain@ximian.com>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -20,6 +34,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <libgnome/gnome-help.h>
 #include <gtk/gtk.h>
+#include <gconf/gconf-client.h>
 
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-control.h>
@@ -41,6 +56,7 @@ static GNOME_Media_CDDBTrackEditor track_editor = CORBA_OBJECT_NIL;
 static gboolean position_auto_update=TRUE;
 static gboolean stopped_state=FALSE;
 static gboolean position_update_ignore_event=FALSE;
+static gboolean remaining_time_mode=FALSE;
 static gchar* discid_prev;
 
 static void
@@ -498,6 +514,8 @@ status_ok (GnomeCD *gcd,
 	int track;
 	char *text;
 	char *current_time;
+
+	gboolean remaining_time_temp = gconf_client_get_bool (gcd->client, "/apps/gnome-cd/show-remaining-time",NULL);
 	
 	/* Allow the track editor to work */
 	gtk_widget_set_sensitive (gcd->trackeditor_b, TRUE);
@@ -538,10 +556,24 @@ status_ok (GnomeCD *gcd,
 		}
 		
 /*  		cd_display_clear (CD_DISPLAY (gcd->display)); */
-		text = g_strdup_printf ("%d:%02d / %d:%02d", 
+		if (remaining_time_temp) {
+			gint min = status->length.minute - status->relative.minute;
+			gint sec = status->length.second - status->relative.second;
+			if(sec < 0) {
+				min--;
+				sec = 60 - (-1 * sec);
+			}
+			text = g_strdup_printf ("- %d:%02d", min, sec);
+			current_time = g_strdup_printf ("Remaining time: %s", text);
+        } else {
+           	text = g_strdup_printf ("%d:%02d / %d:%02d",
 					status->relative.minute, status->relative.second,
 					status->length.minute, status->length.second);
-		current_time = g_strdup_printf ("%d:%02d",status->relative.minute, status->relative.second);
+	 	      		current_time = g_strdup_printf ("Current time: %s", text);
+        }
+
+		/* Displaying tooltip */
+		gtk_tooltips_set_tip (gcd->tooltips, GTK_WIDGET(gcd->display), current_time, NULL);
 		
 		/* update position slider */
 		if(position_auto_update && (status->length.minute!=0 ||
@@ -567,15 +599,15 @@ status_ok (GnomeCD *gcd,
 			
 		/* Update the tray icon tooltip */
 		if (gcd->disc_info != NULL) {
-			text = g_strdup_printf (_("Playing %s - %s\nCurrent Time: %s"),
+			text = g_strdup_printf (_("Playing %s - %s\n%s"),
 						gcd->disc_info->artist ? gcd->disc_info->artist : _("Unknown Artist"),
 						gcd->disc_info->title ? gcd->disc_info->title : _("Unknown Album"),
 						current_time);
 		} else {
-			text = g_strdup_printf (_("Playing\nCurrent Time: %s"), current_time);
+			text = g_strdup_printf (_("Playing\n: %s"), current_time);
 		}
-		if (gcd->tray_tips)	
-			gtk_tooltips_set_tip (gcd->tray_tips, gcd->tray, text, NULL);
+		if (gcd->tray)	
+			gtk_status_icon_set_tooltip (gcd->tray, text);
 		g_free (text);
 		
 		break;
@@ -595,8 +627,8 @@ status_ok (GnomeCD *gcd,
 		set_window_track_title (gcd, status);
 
 		/* Update the tray icon tooltip */
-		if (gcd->tray_tips)	
-			gtk_tooltips_set_tip (gcd->tray_tips, gcd->tray, _("Paused"), NULL);
+		if (gcd->tray)	
+			gtk_status_icon_set_tooltip (gcd->tray, _("Paused"));
 		break;
 		
 	case GNOME_CDROM_AUDIO_COMPLETE:
@@ -646,8 +678,10 @@ status_ok (GnomeCD *gcd,
 			set_track_option_menu (GTK_OPTION_MENU (gcd->tracks), 1);
 
 			/* Update tray icon tooltip */
-			if (gcd->tray_tips)	
-				gtk_tooltips_set_tip (gcd->tray_tips, gcd->tray, _("CD Player"), NULL);
+			if (gcd->tray)	
+				gtk_status_icon_set_tooltip (gcd->tray, _("CD Player"));
+			if (gcd->tooltips)
+				gtk_tooltips_set_tip (gcd->tooltips, GTK_WIDGET(gcd->display), _("CD Player"), NULL);
 
 			/* Initialise the display time to 0:00 in stopped state*/
 			cd_display_set_line (CD_DISPLAY (gcd->display),
@@ -670,8 +704,8 @@ status_ok (GnomeCD *gcd,
 		}
 
 		/* Update the tray icon tooltip */
-		if (gcd->tray_tips)	
-			gtk_tooltips_set_tip (gcd->tray_tips, gcd->tray, _("Stopped"), NULL);
+		if (gcd->tray)	
+			gtk_status_icon_set_tooltip (gcd->tray, _("Stopped"));
 		break;
 		
 	case GNOME_CDROM_AUDIO_ERROR:
@@ -687,8 +721,8 @@ status_ok (GnomeCD *gcd,
 		}
 
 		/* Update the tray icon tooltip */
-		if (gcd->tray_tips)	
-			gtk_tooltips_set_tip (gcd->tray_tips, gcd->tray, _("No disc"), NULL);
+		if (gcd->tray)	
+			gtk_status_icon_set_tooltip (gcd->tray, _("No disc"));
 		break;
 		
 	default:
@@ -871,8 +905,8 @@ cd_status_changed_cb (GnomeCDRom *cdrom,
                 gnome_cd_set_window_title (gcd, NULL, NULL);
 
 		/* Updated the tray icon tooltip */
-		if (gcd->tray_tips)	
-			gtk_tooltips_set_tip (gcd->tray_tips, gcd->tray, _("No Cdrom"), NULL);
+		if (gcd->tray)	
+			gtk_status_icon_set_tooltip (gcd->tray, _("No Cdrom"));
                 break;
 
 	default:
@@ -945,6 +979,15 @@ loopmode_changed_cb (GtkWidget *display,
 	gcd->cdrom->loopmode = mode;
 }
 
+void
+remainingtime_mode_changed_cb (GtkWidget *display,
+                    GnomeCDRomMode mode,
+                    GnomeCD *gcd)
+{
+	remaining_time_mode = !remaining_time_mode;
+	gconf_client_set_bool (gcd->client, "/apps/gnome-cd/show-remaining-time", remaining_time_mode, NULL);
+}
+
 void
 playmode_changed_cb (GtkWidget *display,
 		     GnomeCDRomMode mode,
@@ -974,82 +1017,32 @@ popup_menu_cb (GtkWidget *widget, GnomeCD *gcd)
     return TRUE;
 }
 
-gboolean
-tray_icon_clicked (GtkWidget *widget, GdkEventButton *event, GnomeCD *gcd)
+void
+tray_popup_menu_cb (GtkStatusIcon *icon,
+		    guint          button,
+                    guint          activate_time,
+		    GnomeCD       *gcd)
 {
-	if (event->button != 3) {
-		if (GTK_WIDGET_VISIBLE (gcd->window)) {
-			gtk_widget_hide (gcd->window);
-		} else {
-			gtk_widget_show (gcd->window);
-		}
-
-		return TRUE;
-	} else {
-	if (event->button == 3) {	
-		make_popup_menu(gcd, NULL, TRUE);
-		return TRUE;
-	}
-		return FALSE;
-	}
+    make_popup_menu (gcd, NULL, FALSE);
 }
 
-gboolean
-tray_icon_pressed (GtkWidget *widget, GdkEventKey *event, GnomeCD *gcd)
+void
+tray_icon_activated (GtkStatusIcon *icon, GnomeCD *gcd)
 {
-	if (event->keyval == GDK_space ||
-	    event->keyval == GDK_KP_Space ||
-	    event->keyval == GDK_Return ||
-	    event->keyval == GDK_KP_Enter) {
-		if (GTK_WIDGET_VISIBLE (gcd->window)) {
-			gtk_widget_hide (gcd->window);
-		} else {
-			gtk_widget_show (gcd->window);
-		}
-
-		return TRUE;
+	if (GTK_WIDGET_VISIBLE (gcd->window)) {
+		gtk_widget_hide (gcd->window);
 	} else {
-		return FALSE;
+		gtk_widget_show (gcd->window);
 	}
 }
 
-gint
-tray_icon_expose (GtkWidget* widget, GdkEventExpose *event)
-{
-  /*
-   * Draw focus indication if the GtkEventBox has focus.
-   */
-  if (GTK_WIDGET_HAS_FOCUS (gtk_widget_get_parent (widget)))
-    {
-      gint focus_width, focus_pad;
-      gint x, y, width, height;
-
-      gtk_widget_style_get (widget,
-                            "focus-line-width", &focus_width,
-                            "focus-padding", &focus_pad,
-                            NULL);
-      x = widget->allocation.x + focus_pad;
-      y = widget->allocation.y + focus_pad;
-      width = widget->allocation.width -  2 * focus_pad;
-      height = widget->allocation.height - 2 * focus_pad;
-      gtk_paint_focus (widget->style, widget->window,
-                       GTK_STATE_NORMAL,
-                       &event->area, widget, "button",
-                       x, y, width, height);
-    }
-  return FALSE;
-}
-
 gboolean
 tray_icon_destroyed (GtkWidget *widget, GnomeCD *gcd)
 {
-	GdkPixbuf *pixbuf;
-	GtkWidget *box;
-
 	if (!GTK_WIDGET_VISIBLE (gcd->window)) {
 		gtk_widget_show (gcd->window);
 	}
-	gtk_widget_destroy (gcd->tray);
+	g_object_unref (gcd->tray);
 	gcd->tray = NULL;
 
 	tray_icon_create (gcd);
@@ -1122,7 +1115,6 @@ open_track_editor (GtkWidget *widget,
 	discid = g_strdup_printf ("%08lx", (gulong) data->discid);
 
 	if (discid_prev != NULL) {
-		gint string_compare;
 		gboolean same_cd;
 		same_cd = g_str_equal(discid,discid_prev);
 		if (!same_cd) {
@@ -1192,7 +1184,6 @@ volume_changed (GtkRange *range,
 {
 	double volume;
 	GError *error = NULL;
-	GtkTooltips *volume_level_tooltip;
 	gchar *volume_level;
 	gint scaled_volume;
 
@@ -1205,7 +1196,7 @@ volume_changed (GtkRange *range,
 	scaled_volume = (volume/255)*100;
 	volume_level = g_strdup_printf (_("Volume %d%%"), scaled_volume);
 	gtk_tooltips_set_tip (gcd->tooltips, GTK_WIDGET (gcd->slider), volume_level, NULL);
-	gconf_client_set_int (client, "/apps/gnome-cd/volume", scaled_volume, NULL);
+	gconf_client_set_int (gcd->client, "/apps/gnome-cd/volume", scaled_volume, NULL);
 	g_free (volume_level);
 }
 
